#!/usr/bin/env python

#
# Module manager class.
#

#######################################################################

import threading

#######################################################################

# Get singleton instance.
def getInstance():
    from dm.common.utility.dmModuleManager import DmModuleManager
    try:
        mgr = DmModuleManager()
    except DmModuleManager, ex:
        mgr = ex
    return mgr

class DmModuleManager:
    """ Singleton class used for managing dm modules. """

    # Singleton.
    __instanceLock = threading.RLock()
    __instance = None

    def __init__(self):
        DmModuleManager.__instanceLock.acquire()
        try:
            if DmModuleManager.__instance:
                raise DmModuleManager.__instance
            DmModuleManager.__instance = self
            from dm.common.utility import loggingManager
            self._logger = loggingManager.getLogger(self.__class__.__name__)
            self._lock = threading.RLock()
            self._moduleList = []
            self._modulesRunning = False
        finally:
            DmModuleManager.__instanceLock.release()

    def addModule(self, m):
        self._lock.acquire()
        try:
            self._logger.debug('Adding dm module: %s' % m.__class__.__name__)
            self._moduleList.append(m)
        finally:
            self._lock.release()

    def startModules(self):
        self._lock.acquire()
        try:
            if self._modulesRunning:
                return
            for m in self._moduleList:
                self._logger.debug('Starting dm module: %s' % m.__class__.__name__)
                m.start()
            self._modulesRunning = True
        finally:
            self._lock.release()

    def stopModules(self):
        self._lock.acquire()
        try:
            if not self._modulesRunning:
                return
            n = len(self._moduleList)
            for i in range(0, n):
                m = self._moduleList[n-1-i]
                self._logger.debug('Stopping dm module: %s' % m.__class__.__name__)
                m.stop()
            self._modulesRunning = False
        finally:
            self._lock.release()

#######################################################################
# Testing.

if __name__ == '__main__':
    pass
